(*-------------------------------FileWorks.pas--------------------------
 V1.0.241 - 08.05.2002 current release
------------------------------------------------------------------------
*)

unit FileWorks;

interface

uses Windows, Classes, SysUtils, ShellAPI, StringWorks, FileCtrl;

type
   TFileDateTimeStamp = packed record
      Creation,
      LastAccess,
      LastWrite: TDateTime;
   end;

//V1.0.241
function FileIsOpen(const Filename: String): Boolean;
function CopyDirectory(const Directory, DestinationFolder: String; const GUI, SimpleGUI, CopyConfirmation, MkDirConfirmation, ErrorGUI: Boolean; var UserHasCancelled: Boolean): Boolean;

//V1.0.240
function NTFSCompressFile(const FileName: String; const ForceCompress: Boolean): Boolean;
function NTFSUncompressFile(const FileName: String): Boolean;

//V1.0.239
function GetFileDateTimeStamp(const Filename: String; var FileDateTimeStamp: TFileDateTimeStamp): Boolean;
function SetFileDateTimeStamp(const Filename: String; var FileDateTimeStamp: TFileDateTimeStamp): Boolean;

function CreateDummyFile(Filename: String; FileSize: Int64): Int64;
function DeleteDirectory(Directory: String; CanUndo, DeleteNotEmpty, GUI: Boolean): Boolean;
function DeleteFiles(Path, Mask: string): Boolean;
function DeleteFilesRecursive(Path, Mask: string): Boolean;
function DirIsEmpty(Path: String): Boolean;
function ExtractDriveFromPath(Path: String): String;
function ExtractPureFilename(Name: String): String;
function ExtractFileExtension(Name: String): String;
function FileAttrib(Filename: String; A,H,R,S: Boolean): Boolean;
function FileCopy(Source, Destination: String): Integer;
function GetFileSize(Datei: String): Int64;
function ITB(Directory: String): String;
function ListFiles(Path, Mask: string; ShowPath: boolean): TStringlist;
function ListFilesRecursive(Path, Mask: string; ShowPath: Boolean): TStringList;
//function ListFilesRecursive2(Path, Mask: String; ShowPath: Boolean): TStringList;
function ListFolders(Path: string; ShowPath: boolean): TStringlist;
procedure PatchFile(filename:string;data:array of byte;offset,count:longint);
function RenameDirectory(OldPath, NewPath: String): Boolean;
function MoveDirectory(const Directory, DestinationFolder: String; const GUI, SimpleGUI, MoveConfirmation, MkDirConfirmation, ErrorGUI: Boolean; var UserHasCancelled: Boolean): Boolean;
function ReadStringFromFile(var Filename: String): String;
procedure SaveStringToFile(var Str, Filename: String);



implementation

function FileIsOpen(const Filename: String): Boolean;
var
   Datei: TFileStream;
   Fehler: boolean;
begin
  Fehler:= false;
  if FileExists(Filename) then begin
    try
      Datei:= TFileStream.Create(Filename, fmOpenRead);
    except
      on EFOpenError do Fehler:= true;
    end;
    if Fehler = false then Datei.Free;
  end;
  result:= Fehler;
end;


function CopyDirectory(const Directory, DestinationFolder: String; const GUI, SimpleGUI, CopyConfirmation, MkDirConfirmation, ErrorGUI: Boolean; var UserHasCancelled: Boolean): Boolean;
var
  FOS : TSHFileOpStruct;
  Flags: Word;
begin
   Flags:= 0;
   if GUI then if SimpleGUI then Flags:= Flags or FOF_SIMPLEPROGRESS
          else                   Flags:= Flags or FOF_SILENT;
   if not CopyConfirmation  then Flags:= Flags or FOF_NOCONFIRMATION;
   if not MkDirConfirmation then Flags:= Flags or FOF_NOCONFIRMMKDIR;
   if not ErrorGUI          then Flags:= Flags or FOF_NOERRORUI;
   ZeroMemory(@FOS,SizeOf(FOS));
   with FOS do begin
      wFunc := FO_COPY;
      fFlags := Flags;
      pFrom := PChar(Directory + #0);
      pTo := PChar(DestinationFolder)
   end;
   RESULT := (0 = ShFileOperation(FOS));
   UserHasCancelled:= FOS.fAnyOperationsAborted;
end;

function NTFSCompressFile(const FileName: String; const ForceCompress: Boolean): Boolean;
const
  COMPRESSION_FORMAT_NONE = 0;
  COMPRESSION_FORMAT_DEFAULT = 1;
  FILE_DEVICE_FILE_SYSTEM = 9;
  METHOD_BUFFERED       = 0;
  FILE_READ_DATA        = 1;
  FILE_WRITE_DATA       = 2;
  FSCTL_SET_COMPRESSION = (FILE_DEVICE_FILE_SYSTEM shl 16) or
    ((FILE_READ_DATA or FILE_WRITE_DATA) shl 14) or (16 shl 2) or METHOD_BUFFERED;
var
  hnd:  Integer;
  Comp: SHORT;
  res:  DWORD;
  pcFilename: PChar;
begin
  pcFilename:= PChar(Filename);
  if forceCompress or ((GetFileAttributes(PChar(ExtractFilePath(FileName))) and
    FILE_ATTRIBUTE_COMPRESSED) <> 0) then
  begin
    Result := False;
    if (GetFileAttributes(pcFileName) and FILE_ATTRIBUTE_COMPRESSED) = 0 then
    begin
      hnd := CreateFile(pcFileName, GENERIC_READ or GENERIC_WRITE, 0,nil, OPEN_EXISTING, 0,0);
      try
        Comp := COMPRESSION_FORMAT_DEFAULT;
        if not DeviceIoControl(hnd, FSCTL_SET_COMPRESSION, @Comp,
          SizeOf(SHORT), nil, 0, res, nil) then Exit;
      finally
        CloseHandle(hnd);
      end;
    end;
    Result := True;
  end
  else
    Result := True;
end;

function NTFSUncompressFile(const FileName: String): Boolean;
const
  COMPRESSION_FORMAT_NONE = 0;
  COMPRESSION_FORMAT_DEFAULT = 1;
  FILE_DEVICE_FILE_SYSTEM = 9;
  METHOD_BUFFERED       = 0;
  FILE_READ_DATA        = 1;
  FILE_WRITE_DATA       = 2;
  FSCTL_SET_COMPRESSION = (FILE_DEVICE_FILE_SYSTEM shl 16) or
    ((FILE_READ_DATA or FILE_WRITE_DATA) shl 14) or (16 shl 2) or METHOD_BUFFERED;
var
  hnd:  Integer;
  Comp: SHORT;
  res:  DWORD;
  pcFilename: PChar;
begin
  pcFilename:= PChar(Filename);
  Result := False;
  if (GetFileAttributes(pcFileName) and FILE_ATTRIBUTE_COMPRESSED) <> 0 then
  begin
    hnd := CreateFile(pcFileName, GENERIC_READ or GENERIC_WRITE, 0,nil, OPEN_EXISTING, 0,0);
    try
      Comp := COMPRESSION_FORMAT_NONE;
      if not DeviceIoControl(hnd, FSCTL_SET_COMPRESSION, @Comp,
        SizeOf(SHORT), nil, 0, res, nil) then Exit;
    finally
      CloseHandle(hnd);
    end;
    Result := True;
  end
  else
    Result := True;
end;

function GetFileDateTimeStamp(const Filename: String; var FileDateTimeStamp: TFileDateTimeStamp): Boolean;
var
   hFile: THandle;
   ftCreation,
   ftLastAccess,
   ftLastWrite,
   ftlCreation,
   ftlLastAccess,
   ftlLastWrite: TFileTime;
   stCreation,
   stLastAccess,
   stLastWrite: TSystemTime;
begin
   result:= FALSE;
   with FileDateTimeStamp do begin
      Creation:= 0;
      LastAccess:= 0;
      LastWrite:= 0;
   end;
   hFile:= CreateFile(PChar(Filename),
                      GENERIC_READ or GENERIC_WRITE,
                      FILE_SHARE_READ,
                      nil,
                      OPEN_EXISTING,
                      FILE_FLAG_BACKUP_SEMANTICS
                      or FILE_ATTRIBUTE_DIRECTORY, 0);
   if (hFile = INVALID_HANDLE_VALUE) then begin
      CloseHandle(hFile);
      exit;
   end else
   try
      if GetFileTime(hFile, @ftCreation,
                            @ftLastAccess,
                            @ftLastWrite) then begin
         if  FileTimeToLocalFileTime(ftCreation, ftlCreation)
         and FileTimeToLocalFileTime(ftLastAccess, ftlLastAccess)
         and FileTimeToLocalFileTime(ftLastWrite, ftlLastWrite) then begin
            if  FileTimeToSystemTime(ftlCreation, stCreation)
            and FileTimeToSystemTime(ftlLastAccess, stLastAccess)
            and FileTimeToSystemTime(ftlLastWrite, stLastWrite) then begin
               FileDateTimeStamp.Creation:= SystemTimeToDateTime(stCreation);
               FileDateTimeStamp.LastAccess:= SystemTimeToDateTime(stLastAccess);
               FileDateTimeStamp.LastWrite:= SystemTimeToDateTime(stLastWrite);
               result:= TRUE;
            end;
         end;
      end;
   finally
      CloseHandle(hFile);
   end;
end;

function SetFileDateTimeStamp(const Filename: String; var FileDateTimeStamp: TFileDateTimeStamp): Boolean;
var
   hFile: THandle;
   ftCreation,
   ftLastAccess,
   ftLastWrite,
   ftlCreation,
   ftlLastAccess,
   ftlLastWrite: TFileTime;
   stCreation,
   stLastAccess,
   stLastWrite: TSystemTime;
begin
   result:= FALSE;
   with FileDateTimeStamp do begin
      Creation:= 0;
      LastAccess:= 0;
      LastWrite:= 0;
   end;
   hFile:= CreateFile(PChar(Filename),
                      GENERIC_READ or GENERIC_WRITE,
                      FILE_SHARE_READ,
                      nil,
                      OPEN_EXISTING,
                      FILE_FLAG_BACKUP_SEMANTICS
                      or FILE_ATTRIBUTE_DIRECTORY, 0);
   if (hFile = INVALID_HANDLE_VALUE) then begin
      CloseHandle(hFile);
      exit;
   end else
   try
      with FileDateTimeStamp do begin
         DateTimeToSystemTime(Creation, stCreation);
         DateTimeToSystemTime(LastAccess, stLastAccess);
         DateTimeToSystemTime(LastWrite, stLastWrite);
         if SystemTimeToFileTime(stCreation, ftlCreation) and
            SystemTimeToFileTime(stLastAccess, ftlLastAccess) and
            SystemTimeToFileTime(stLastWrite, ftlLastWrite) then begin
            if LocalFileTimeToFileTime(ftlCreation, ftCreation) and
               LocalFileTimeToFileTime(ftlLastAccess, ftLastAccess) and
               LocalFileTimeToFileTime(ftlLastWrite, ftLastWrite) then begin
               result:= SetFileTime(hFile, @ftCreation, @ftLastAccess, @ftLastWrite);
            end;
         end;
      end;
   finally
      CloseHandle(hFile);
   end;
end;

function CreateDummyFile(Filename: String; FileSize: Int64): Int64;
var
   FS: TFileStream;
   I: Integer;
   FillChr: Char;
   SizeF: Integer;
begin
   result:= -1;
   SizeF:= 1;
   randomize;
   if FileExists(Filename) then exit;
   FS:= TFileStream.Create(Filename, fmCreate);
   for I:=0 to FileSize - 1 do begin
      FillChr:= Chr(I-Random(255)+1);
      FS.Write(FillChr ,SizeF);
   end;
   FS.Free;
end;

function ExtractPureFilename(Name: String): String;
var
   Dots, DotPos: Integer;
begin
   Dots:= StringCountInStr('.', Name);
   DotPos:= SubPositionByIndex(Name, '.', Dots);
   result:= StrLeft(Name, DotPos - 1);
end;

function ExtractFileExtension(Name: String): String;
var
   Dots, DotPos, StrLn: Integer;
begin
   Dots:= StringCountInStr('.', Name);
   StrLn:= StrLen(PChar(Name));
   DotPos:= SubPositionByIndex(Name, '.', Dots);
   result:= Copy(Name, DotPos + 1, StrLn);
end;

function FileCopy(Source, Destination: String): Integer;
var
  Q, Z: TFileStream;
begin
   if not FileExists(Source) then begin
      result:= -2; (* Fehlercode fr "Quelle nicht gefunden" *)
      exit;
   end;
   if FileExists(Destination) then begin
      result:= -3; (* Fehlercode fr "Zieldatei vorhanden" *)
      exit;
   end;
   try
      Q := TFileStream.Create(Source, fmOpenRead);
      Z := TFileStream.Create(Destination, fmCreate);
      Z.CopyFrom(Q, Q.Size);
      Q.Free;
      Z.Free;
      result:= 0; (* Fehlercode 0 fr erfolgreiche Ausfhrung *)
   except
      result:= -4; (* Fehlercode fr "Datei konnte nicht kopiert werden" *)
   end;
end;

function FileMove(Source, Destination: String): Integer;
var
  Q, Z: TFileStream;
begin
   if not FileExists(Source) then begin
      result:= -2; (* Fehlercode fr "Quelle nicht gefunden" *)
      exit;
   end;
   if FileExists(Destination) then begin
      result:= -3; (* Fehlercode fr "Zieldatei vorhanden" *)
      exit;
   end;
   try
      Q := TFileStream.Create(Source, fmOpenRead);
      Z := TFileStream.Create(Destination, fmCreate);
      Z.CopyFrom(Q, Q.Size);
      Q.Free;
      Z.Free;
      if not DeleteFile(PChar(Source)) then begin
         result:= -5; (* Fehlercode fr "Quelldatei kann nicht entfernt werden" *)
         exit;
      end;
   result:= 0; (* Fehlercode 0 fr erfolgreiche Ausfhrung *)
   except
      result:= -4; (* Fehlercode fr "Datei konnte nicht kopiert werden" *)
   end;
end;

function ListFiles(Path, Mask: string; ShowPath: boolean): TStringlist;
var
  SRec: TSearchRec;
  SL: TStringList;
begin
   (* ++++++++++++++++ Code sample by Pumi ++++++++++++++++++ *)
   SL := TStringList.Create;
   FindFirst(Path + Mask, not faDirectory, SRec);
   if ShowPath then
      SL.Add(Path + SRec.Name)
   else
      SL.Add(SRec.Name);
   while FindNext(SRec) = 0 do
      if ShowPath then
         SL.Add(Path + SRec.Name)
      else
         SL.Add(SRec.Name);
   FindClose(SRec);
   Result := SL;
end;

function ListFolders(Path: string; ShowPath: boolean): TStringlist;
var
  SRec: TSearchRec;
  SL: TStringList;
begin
   (* ++++++++++++++++ Code sample by Pumi ++++++++++++++++++ *)
   SL := TStringList.Create;
   result:= TStringList.Create;   
   FindFirst(Path + '*.*', {not faAnyFile +
                           not faSysFile +
                           not faHidden +
                           not faReadOnly}
                           faDirectory,
                           SRec);
   if ShowPath then
      SL.Add(Path + SRec.Name)
   else
      SL.Add(SRec.Name);
   while FindNext(SRec) = 0 do
      if ShowPath then
         SL.Add(Path + SRec.Name)
      else
         SL.Add(SRec.Name);
   FindClose(SRec);
   Result.Assign(SL);
end;

function DeleteFiles(Path, Mask: string): Boolean;
var
  SRec: TSearchRec;
  SL: TStringList;
  I: Integer;
begin
   (* ++++++++++++++++ Code based on Assarbad ++++++++++++++++++ *)
   result:= FALSE;
   SL := TStringList.Create;
   FindFirst(Path + Mask, not faDirectory, SRec);
   SL.Add(Path + SRec.Name);
   while FindNext(SRec) = 0 do SL.Add(Path + SRec.Name);
   FindClose(SRec);
   if SL.Count < 1 then exit;
   for I:= 0 to SL.Count - 1 do begin
      DeleteFile(PChar(SL[I]));
   end;
   result:= TRUE;
end;

{ -------- This version has a bug in recursive search---------
function ListFilesRecursive(Path, Mask: string; ShowPath: Boolean): TStringList;
var SR: TSearchRec;
    err, I: Integer;
    dummy: String;
    TempList: TStringList;
begin
   (* ++++++++++++++++ Code sample by Assarbad ++++++++++++++++++ *)
     err:=FindFirst(Path+Mask, faAnyFile, SR);
     Path:= IncludeTrailingBackslash(Path);
     result:= TStringList.Create;
     TempList:= TStringList.Create;
     while err=0 do begin
           if (sr.name<>'.') and (sr.name<>'..') then
              if ((sr.Attr and fadirectory)<>0) then begin
                 dummy:=path+sr.name+'\';
                 TempList.Assign(ListFilesRecursive(dummy, mask, ShowPath));
                 for I:= 0 to TempList.Count - 1 do begin
//                    if ShowPath then result.Add(Dummy + TempList[I]) else
                       result.Add(TempList[I]);
                 end;
              end else if ShowPath then result.Append(Path + sr.name) else
                 result.Append(sr.name);
           err:=findnext(sr);
     end;
     FindClose(SR);
     TempList.Free;
end;}

function ListFilesRecursive(Path, Mask: String; ShowPath: Boolean): TStringList;
var SR: TSearchRec;
    Erg, I: Integer;
    RelDir, TempStr : String;
    TempList: TStringList;
begin
   IncludeTrailingBackslash(Path);
   RelDir := ExtractFilePath(Mask);
   Mask := ExtractFileName(Mask);
   result:= TStringList.Create;
   TempList:= TStringList.Create;
   Erg := FindFirst(Path+RelDir+'*.*', faDirectory, SR);
   while Erg = 0 do
   begin
      if (SR.attr and faDirectory) <> 0 then
         if SR.Name[1] <> '.' then begin
            TempList.Assign(ListFilesRecursive(Path, RelDir+SR.Name+'\'+Mask, ShowPath));
            for I:= 0 to TempList.Count - 1 do begin
               TempStr:= TempList[I];
               if ShowPath and not (TempStr[2] = ':') then TempStr:= Path + TempStr;
               result.Add(TempList[I]);
            end;
         end;
      Erg := FindNext(SR);
   end;
   Erg := FindFirst(Path+RelDir+Mask, $27, SR);
   {Das Attribut $27 bedeutet
   alle echten Dateien}
   while Erg = 0 do
   begin
      if not ShowPath then
         result.Add(RelDir+SR.Name)
      else
      result.Add(Path+RelDir+SR.Name);
      Erg := FindNext(SR);
   end;
   FindClose(SR);
   TempList.Free;
end;

function DeleteFilesRecursive(Path, Mask: string): Boolean;
var SR: TSearchRec;
    err, I:integer;
    dummy:string;
    SL: TStringList;
begin
   (* ++++++++++++++++ Code based on Assarbad ++++++++++++++++++ *)
     result:= FALSE;
     err:=FindFirst(Path+Mask, faAnyFile, SR);
     SL:= TStringList.Create;
     while err=0 do begin
           if (sr.name<>'.') and (sr.name<>'..') then
              if ((sr.Attr and fadirectory)<>0) then begin
                 dummy:=path+sr.name+'\';
                 ListFilesRecursive(dummy, Mask, TRUE);
              end else SL.Append(sr.name);
           err:=findnext(sr);
     end;
     FindClose(SR);
     if SL.Count < 1 then exit;
     for I:= 0 to SL.Count - 1 do begin
        DeleteFile(PChar(SL[I]));
     end;
     result:= TRUE;
end;

function DirIsEmpty(Path: String): Boolean;
var
   SL: TStringList;
begin
   SL:= ListFilesRecursive(ITB(Path), '*.*', TRUE);
   result:= (SL.Count < 1);
end;

function ExtractDriveFromPath(Path: String): String;
begin
   if (StrLen(PChar(Path)) > 0) then result:= Copy(Path, 1, 1) else result:= '';
end;

function ITB(Directory: String): String;
begin
   result:= UpperCase(IncludeTrailingBackslash(Directory));
end;

function RenameDirectory(OldPath, NewPath: String): Boolean;
begin
   result:= FALSE;
   if not (ExtractDriveFromPath(OldPath) = ExtractDriveFromPath(NewPath)) then
      exit;
   result:= MoveFile(PChar(OldPath), PChar(NewPath));
end;

function MoveDirectory(const Directory, DestinationFolder: String; const GUI, SimpleGUI, MoveConfirmation, MkDirConfirmation, ErrorGUI: Boolean; var UserHasCancelled: Boolean): Boolean;
var
  FOS : TSHFileOpStruct;
  Flags: Word;
begin
   Flags:= 0;
   if GUI then if SimpleGUI then Flags:= Flags or FOF_SIMPLEPROGRESS
          else                   Flags:= Flags or FOF_SILENT;
   if not MoveConfirmation  then Flags:= Flags or FOF_NOCONFIRMATION;
   if not MkDirConfirmation then Flags:= Flags or FOF_NOCONFIRMMKDIR;
   if not ErrorGUI          then Flags:= Flags or FOF_NOERRORUI;
   ZeroMemory(@FOS,SizeOf(FOS));
   with FOS do begin
      wFunc := FO_MOVE;
      fFlags := Flags;
      pFrom := PChar(Directory + #0);
      pTo := PChar(DestinationFolder)
   end;
   RESULT := (0 = ShFileOperation(FOS));
   UserHasCancelled:= FOS.fAnyOperationsAborted;
end;

function DeleteDirectory(Directory: String; CanUndo, DeleteNotEmpty, GUI: Boolean): Boolean;
var
  FOS : TSHFileOpStruct;
  FOS_FLAG: DWORD;
begin
   result:= FALSE;
   Directory:= ITB(Directory);
   if not DirectoryExists(Directory) then exit;
   if not DeleteNotEmpty then begin
      if not DirIsEmpty(Directory) then exit;
   end;
   ZeroMemory(@FOS,SizeOf(FOS));
   if CanUndo then FOS_FLAG:= FOF_ALLOWUNDO else FOS_FLAG:= 0;
   if not GUI then FOS_FLAG:= FOS_FLAG + FOF_NOCONFIRMATION;
   with FOS do begin
      wFunc := FO_DELETE;
      fFlags := FOS_FLAG;
      pFrom := PChar(Directory + #0);
   end;
   RESULT := (0 = ShFileOperation(FOS));
end;

function  GetFileSize(Datei: String): Int64;
var
   TempFileSize: Int64;
   SR: TSearchRec;
begin
     TempFileSize:= 0;
     if FindFirst(Datei,faAnyFile,SR) = 0 then
     TempFileSize:= SR.Size;
     FindClose(SR);
     result:= TempFileSize;
end;

procedure SaveStringToFile(var Str, Filename: String);
var
   TF: TextFile;
begin
   AssignFile(TF, Filename);
   Rewrite(TF);
   Write(TF, Str);
   Close(TF);
end;

function ReadStringFromFile(var Filename: String): String;
var
   TF: TextFile;
begin
   if not FileExists(Filename) then begin
      result:= '';
      exit;
   end;
   AssignFile(TF, Filename);
   Reset(TF);
   Read(TF, result);
   Close(TF);
end;

function FileAttrib(Filename: String; A,H,R,S: Boolean): Boolean;
var
   Attrbs: DWord;
begin
   Attrbs:= 0;
   if A then Attrbs:= Attrbs and FILE_ATTRIBUTE_ARCHIVE;
   if H then Attrbs:= Attrbs and FILE_ATTRIBUTE_HIDDEN;
   if R then Attrbs:= Attrbs and FILE_ATTRIBUTE_READONLY;
   if S then Attrbs:= Attrbs and FILE_ATTRIBUTE_SYSTEM;
   result:= SetFileAttributes(PChar(Filename), Attrbs);
end;
procedure PatchFile(filename:string;data:array of byte;offset,count:longint);
var
  f:file;
begin
  assignfile(f,filename);
  {$i-}reset(f,1);{$i+}if ioresult<>0 then exit;
  seek(f,offset);
  blockwrite(f,data,count);
  closefile(f);
end;




end.
