(*---------------------------------RegWorks.pas--------------------------
 V1.0.236 - 10.01.2001 current release
------------------------------------------------------------------------*)
unit RegWorks;

interface

uses Windows, Graphics, Registry, SysUtils, Classes, StringWorks;

type
   THostInfo = packed record
     Host:           String;
     Port:           Integer;
   end;

   TSysProxy = packed record
     FTP:            THostInfo;
     GOPHER:         THostInfo;
     HTTP:           THostInfo;
     HTTPS:          THostInfo;
     SOCKS:          THostInfo;
   end;

   REGW_INTERNETPROXY = class(TObject)
   private
      { Private-Deklarationen }
      function pIsProxyUse: Boolean;
      function pProxyString: String;
      function pProcessProxyString(ProxyString: String): TSysProxy;
   public
      { Public-Deklarationen }
      function Used:  Boolean;
      function Data: TSysProxy;
   end;

// Added rev. 1.0.235 / 27.12.2001
function HKEYToStr(KEY: HKEY): String;
function StrToHKEY(KEY: String): HKEY;

// Added rev. 1.0.236 / 10.01.2002
function RegColorToStr(RegColor: String): String;
function ColorToRegColor(Color: TColor): String;
function ReadRegColor(RootKey: HKEY; KeyName, ValueName: String): TColor;
function WriteRegColor(Color: TColor; RootKey: HKEY; KeyName, ValueName: String): Boolean;

implementation

function REGW_INTERNETPROXY.pIsProxyUse: Boolean;
var
   Reg: TRegistry;
begin
   result:= FALSE;
   Reg:= TRegistry.Create(KEY_READ);
   with Reg do begin
      if OpenKey('Software\Microsoft\Windows\CurrentVersion\Internet settings',
         False) then result:= ReadBool('ProxyEnable');
      Free;
   end;
end;

function REGW_INTERNETPROXY.pProxyString: String;
var
   Reg: TRegistry;
begin
   result:= '';
   Reg:= TRegistry.Create(KEY_READ);
   with Reg do begin
      if OpenKey('Software\Microsoft\Windows\CurrentVersion\Internet settings',
         False) then result:= ReadString('ProxyServer');
      Free;
   end;
end;

function REGW_INTERNETPROXY.pProcessProxyString(ProxyString: String): TSysProxy;
var
   HostsPortList, HostPort: TStringList;
begin
   HostsPortList:= StrToList(ProxyString, ';');
   HostPort:= StrToList(HostsPortList[0], ':');
   result.FTP.Host:= StrToList(HostPort[0], '=')[1];
   result.FTP.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[1], ':');
   result.GOPHER.Host:= StrToList(HostPort[0], '=')[1];
   result.GOPHER.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[2], ':');
   result.HTTP.Host:= StrToList(HostPort[0], '=')[1];
   result.HTTP.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[3], ':');
   result.HTTPS.Host:= StrToList(HostPort[0], '=')[1];
   result.HTTPS.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[4], ':');
   result.SOCKS.Host:= StrToList(HostPort[0], '=')[1];
   result.SOCKS.Port:= StrToIntDef(HostPort[1], 0);
end;

function REGW_INTERNETPROXY.Used:  Boolean;
begin
   result:= pIsProxyUse;
end;

function REGW_INTERNETPROXY.Data: TSysProxy;
begin
   result:= pProcessProxyString(pProxyString);
end;

function HKEYToStr(KEY: HKEY): String;
begin
   case KEY of
   HKEY_CLASSES_ROOT: result:= 'HKEY_CLASSES_ROOT';
   HKEY_CURRENT_USER: result:= 'HKEY_CURRENT_USER';
   HKEY_LOCAL_MACHINE: result:= 'HKEY_LOCAL_MACHINE';
   HKEY_USERS: result:= 'HKEY_USERS';
   HKEY_PERFORMANCE_DATA: result:= 'HKEY_PERFORMANCE_DATA';
   HKEY_CURRENT_CONFIG: result:= 'HKEY_CURRENT_CONFIG';
   HKEY_DYN_DATA: result:= 'HKEY_DYN_DATA';
   else result:= 'HKEY_LOCAL_MACHINE';
   end;
end;

function StrToHKEY(KEY: String): HKEY;
begin
   if KEY = 'HKEY_CLASSES_ROOT' then result:= HKEY_CLASSES_ROOT else
   if KEY = 'HKEY_CURRENT_USER' then result:= HKEY_CURRENT_USER else
   if KEY = 'HKEY_LOCAL_MACHINE' then result:= HKEY_LOCAL_MACHINE else
   if KEY = 'HKEY_USERS' then result:= HKEY_USERS else
   if KEY = 'HKEY_PERFORMANCE_DATA' then result:= HKEY_PERFORMANCE_DATA else
   if KEY = 'HKEY_CURRENT_CONFIG' then result:= HKEY_CURRENT_CONFIG else
   if KEY = 'HKEY_DYN_DATA' then result:= HKEY_DYN_DATA else
   result:= HKEY_LOCAL_MACHINE;
end;

function RegColorToStr(RegColor: String): String;
var
   TempList: TStringList;
   sR, sG, sB: String;
begin
   result:= '';
   if CountCharInStr(RegColor, ' ') <> 2 then exit;
   if (StringLen(RegColor) > 11) or
      (StringLen(RegColor) <  5) then exit;
   TempList:= TStringList.Create;
   TempList.Assign(StrToList(RegColor, ' '));
   sR:= TempList[0];
   sG:= TempList[1];
   sB:= TempList[2];
   result:= '$00' +
            IntToHex(StrToIntDef(sB, 0), 2) +
            IntToHex(StrToIntDef(sG, 0), 2) +
            IntToHex(StrToIntDef(sR, 0), 2);
   TempList.Free;
end;

function ColorToRegColor(Color: TColor): String;
var
   clStr, sR, sG, sB: String;
const
   spc: Char = ' ';
   dlr: Char = '$';
begin
   clStr:= IntToHex(Color, 6);
   sR:= StrRight(clStr, 2);
   sG:= StrMid(clStr, 3, 2);
   sB:= StrMid(clStr, 2, 2);
   sR:= IntToStr(StrToInt(dlr + sR));
   sG:= IntToStr(StrToInt(dlr + sG));
   sB:= IntToStr(StrToInt(dlr + sB));
   result:= sR + spc + sG + spc + sB;
end;

function ReadRegColor(RootKey: HKEY; KeyName, ValueName: String): TColor;
var
   Reg: TRegistry;
   DataInfo: TRegDataInfo;
begin
   result:= clNone;
   Reg:= TRegistry.Create(KEY_READ);
   Reg.RootKey:= RootKey;
   with Reg do begin
      if OpenKey(KeyName, FALSE) then begin
         GetDataInfo(ValueName, DataInfo);
         if (DataInfo.RegData = rdString) then begin
            result:= StringToColor(RegColorToStr(ReadString(ValueName)));
         end;
      end;
      Free;
   end;
end;

function WriteRegColor(Color: TColor; RootKey: HKEY; KeyName, ValueName: String): Boolean;
var
   Reg: TRegistry;
begin
   result:= FALSE;
   Reg:= TRegistry.Create;
   with Reg do begin
      RootKey:= RootKey;
      if OpenKey(KeyName, TRUE) then begin
         WriteString(ValueName, ColorToRegColor(Color));
         result:= TRUE;
      end;
      Free;
   end;
end;

end.

